@@ -4,6 +4,7 @@ import autograd
 from scipy.optimize import minimize
 
 import geomstats.backend as gs
+import geomstats.error
 from geomstats.integrator import integrate
 
 
@@ -11,29 +12,37 @@ N_STEPS = 10
 EPSILON = 1e-3
 
 
-class Connection(object):
+class Connection:
     """Class for affine connections.
 
     Parameters
     ----------
-    dimension: int
+    dim: int
         Dimension of the underlying manifold.
     """
 
-    def __init__(self, dimension):
-        self.dimension = dimension
+    def __init__(
+            self, dim, default_point_type='vector',
+            default_coords_type='intrinsic'):
+        geomstats.error.check_integer(dim, 'dim')
+        geomstats.error.check_parameter_accepted_values(
+            default_point_type, 'default_point_type', ['vector', 'matrix'])
+
+        self.dim = dim
+        self.default_point_type = default_point_type
+        self.default_coords_type = default_coords_type
 
     def christoffels(self, base_point):
         """Christoffel symbols associated with the connection.
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dimension]
+        base_point : array-like, shape=[n_samples, dim]
             Point on the manifold.
 
         Returns
         -------
-        gamma : array-like, shape=[n_samples, dimension, dimension, dimension]
+        gamma : array-like, shape=[n_samples, dim, dim, dim]
             Values of the christoffel symbols, with the covariant index on
             the first dimension.
         """
@@ -48,11 +57,11 @@ class Connection(object):
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, dimension]
+        tangent_vec_a : array-like, shape=[n_samples, dim]
             Tangent vector at base point.
-        tangent_vec_b : array-like, shape=[n_samples, dimension]
+        tangent_vec_b : array-like, shape=[n_samples, dim]
             Tangent vector at base point.
-        base_point : array-like, shape=[n_samples, dimension]
+        base_point : array-like, shape=[n_samples, dim]
             Point on the manifold.
         """
         raise NotImplementedError(
@@ -63,22 +72,23 @@ class Connection(object):
 
         Parameters
         ----------
-        velocity : array-like, shape=[n_samples, dimension]
+        velocity : array-like, shape=[n_samples, dim]
             Tangent vector at the position.
-        position : array-like, shape=[n_samples, dimension]
+        position : array-like, shape=[n_samples, dim]
             Point on the manifold, the position at which to compute the
             geodesic ODE.
 
         Returns
         -------
-        geodesic_ode : array-like, shape=[n_samples, dimension]
+        geodesic_ode : array-like, shape=[n_samples, dim]
             Value of the vector field to be integrated at position.
         """
         gamma = self.christoffels(position)
         return - gs.einsum('...kij,...i,...j->...k', gamma, velocity,
                            velocity)
 
-    def exp(self, tangent_vec, base_point, n_steps=N_STEPS, step='euler'):
+    def exp(self, tangent_vec, base_point, n_steps=N_STEPS, step='euler',
+            point_type=None):
         """Exponential map associated to the affine connection.
 
         Exponential map at base_point of tangent_vec computed by integration
@@ -87,18 +97,20 @@ class Connection(object):
 
         Parameters
         ----------
-        tangent_vec : array-like, shape=[n_samples, dimension]
+        tangent_vec : array-like, shape=[n_samples, dim]
             Tangent vector at the base point.
-        base_point : array-like, shape=[n_samples, dimension]
+        base_point : array-like, shape=[n_samples, dim]
             Point on the manifold.
         n_steps : int
             The number of discrete time steps to take in the integration.
         step : str, {'euler', 'rk4'}
             The numerical scheme to use for integration.
+        point_type : str, {'vector', 'matrix'}
+            Type of representation used for points.
 
         Returns
         -------
-        exp : array-like, shape=[n_samples, dimension]
+        exp : array-like, shape=[n_samples, dim]
             Point on the manifold.
         """
         tangent_vec = gs.to_ndarray(tangent_vec, to_ndim=2)
@@ -116,9 +128,9 @@ class Connection(object):
 
         Parameters
         ----------
-        point : array-like, shape=[n_samples, dimension]
+        point : array-like, shape=[n_samples, dim]
             Point on the manifold.
-        base_point : array-like, shape=[n_samples, dimension]
+        base_point : array-like, shape=[n_samples, dim]
             Point on the manifold.
         n_steps : int
             The number of discrete time steps to take in the integration.
@@ -127,7 +139,7 @@ class Connection(object):
 
         Returns
         -------
-        tangent_vec : array-like, shape=[n_samples, dimension]
+        tangent_vec : array-like, shape=[n_samples, dim]
             Tangent vector at the base point.
         """
         point = gs.to_ndarray(point, to_ndim=2)
@@ -138,7 +150,7 @@ class Connection(object):
             """Define the objective function."""
             velocity = velocity.reshape(base_point.shape)
             delta = self.exp(velocity, base_point, n_steps, step) - point
-            loss = 1. / self.dimension * gs.sum(delta ** 2, axis=1)
+            loss = 1. / self.dim * gs.sum(delta ** 2, axis=1)
             return 1. / n_samples * gs.sum(loss)
 
         objective_grad = autograd.elementwise_grad(objective)
@@ -147,7 +159,7 @@ class Connection(object):
             """Create helpful objective func wrapper for autograd comp."""
             return objective(velocity), objective_grad(velocity)
 
-        tangent_vec = gs.random.rand(base_point.size)
+        tangent_vec = gs.random.rand(gs.sum(gs.shape(base_point)))
         res = minimize(
             objective_with_grad, tangent_vec, method='L-BFGS-B', jac=True,
             options={'disp': False, 'maxiter': 25})
@@ -165,11 +177,11 @@ class Connection(object):
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dimension]
+        base_point : array-like, shape=[n_samples, dim]
             Point on the manifold, from which to transport.
-        next_point : array-like, shape=[n_samples, dimension]
+        next_point : array-like, shape=[n_samples, dim]
             Point on the manifold, to transport to.
-        base_shoot : array-like, shape=[n_samples, dimension]
+        base_shoot : array-like, shape=[n_samples, dim]
             Point on the manifold, end point of the geodesics starting
             from the base point with initial speed to be transported.
         return_geodesics : bool, optional (defaults to False)
@@ -179,9 +191,9 @@ class Connection(object):
         Returns
         -------
         next_step : dict of array-like and callable with following keys:
-            next_tangent_vec : array-like, shape=[n_samples, dimension]
+            next_tangent_vec : array-like, shape=[n_samples, dim]
                 Tangent vector at end point.
-            end_point : array-like, shape=[n_samples, dimension]
+            end_point : array-like, shape=[n_samples, dim]
                 Point on the manifold, closes the geodesic parallelogram of the
                 construction.
             geodesics : list of callable, len=3 (only if
@@ -243,11 +255,11 @@ class Connection(object):
 
         Parameters
         ----------
-        base_point : array-like, shape=[n_samples, dimension]
+        base_point : array-like, shape=[n_samples, dim]
             Point on the manifold, from which to transport.
-        next_point : array-like, shape=[n_samples, dimension]
+        next_point : array-like, shape=[n_samples, dim]
             Point on the manifold, to transport to.
-        base_shoot : array-like, shape=[n_samples, dimension]
+        base_shoot : array-like, shape=[n_samples, dim]
             Point on the manifold, end point of the geodesics starting
             from the base point with initial speed to be transported.
         return_geodesics : bool, optional (defaults to False)
@@ -256,9 +268,9 @@ class Connection(object):
 
         Returns
         -------
-        transported_tangent_vector : array-like, shape=[n_samples, dimension]
+        transported_tangent_vector : array-like, shape=[n_samples, dim]
             Tangent vector at end point.
-        end_point : array-like, shape=[n_samples, dimension]
+        end_point : array-like, shape=[n_samples, dim]
             Point on the manifold, closes the geodesic parallelogram of the
             construction.
 
@@ -326,12 +338,12 @@ class Connection(object):
 
         Parameters
         ----------
-        tangent_vec_a : array-like, shape=[n_samples, dimension]
+        tangent_vec_a : array-like, shape=[n_samples, dim]
             Tangent vector at base point to transport.
-        tangent_vec_b : array-like, shape=[n_samples, dimension]
+        tangent_vec_b : array-like, shape=[n_samples, dim]
             Tangent vector at base point, initial speed of the geodesic along
             which to transport.
-        base_point : array-like, shape=[n_samples, dimension]
+        base_point : array-like, shape=[n_samples, dim]
             Point on the manifold, initial position of the geodesic along
             which to transport.
         n_steps : int
@@ -393,7 +405,7 @@ class Connection(object):
 
         Parameters
         ----------
-        base_point: array-like, shape=[n_samples, dimension]
+        base_point: array-like, shape=[n_samples, dim]
             Point on the manifold.
         """
         raise NotImplementedError(
@@ -410,12 +422,12 @@ class Connection(object):
 
         Parameters
         ----------
-        initial_point : array-like, shape=[n_samples, dimension]
+        initial_point : array-like, shape=[n_samples, dim]
             Point on the manifold, initial point of the geodesic.
-        end_point : array-like, shape=[n_samples, dimension], optional
+        end_point : array-like, shape=[n_samples, dim], optional
             Point on the manifold, end point of the geodesic. If None,
             an initial tangent vector must be given.
-        initial_tangent_vec : array-like, shape=[n_samples, dimension],
+        initial_tangent_vec : array-like, shape=[n_samples, dim],
             optional
             Tangent vector at base point, the initial speed of the geodesics.
             If None, an end point must be given and a logarithm is computed.
@@ -443,7 +455,10 @@ class Connection(object):
                 point=end_point,
                 base_point=initial_point)
             if initial_tangent_vec is not None:
-                assert gs.allclose(shooting_tangent_vec, initial_tangent_vec)
+                if not gs.allclose(shooting_tangent_vec, initial_tangent_vec):
+                    raise RuntimeError(
+                        'The shooting tangent vector is too'
+                        ' far from the initial tangent vector.')
             initial_tangent_vec = shooting_tangent_vec
         initial_tangent_vec = gs.array(initial_tangent_vec)
         initial_tangent_vec = gs.to_ndarray(
@@ -491,7 +506,7 @@ class Connection(object):
 
         Parameters
         ----------
-        base_point: array-like, shape=[n_samples, dimension]
+        base_point: array-like, shape=[n_samples, dim]
             Point on the manifold.
         """
         raise NotImplementedError(
