@@ -6,12 +6,14 @@ import tests.helper as helper
 
 import geomstats.backend as gs
 import geomstats.tests
-from geomstats.geometry.hyperbolic import Hyperbolic
+from geomstats.geometry.hyperboloid import Hyperboloid
 from geomstats.geometry.minkowski import Minkowski
+from geomstats.geometry.poincare_ball import PoincareBall
 
 # Tolerance for errors on predicted vectors, relative to the *norm*
 # of the vector, as opposed to the standard behavior of gs.allclose
 # where it is relative to each element of the array
+
 RTOL = 1e-6
 
 
@@ -19,21 +21,42 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
     def setUp(self):
         gs.random.seed(1234)
         self.dimension = 3
-        self.space = Hyperbolic(dimension=self.dimension)
+        self.space = Hyperboloid(dim=self.dimension)
         self.metric = self.space.metric
+        self.ball_manifold = PoincareBall(dim=2)
         self.n_samples = 10
 
     def test_random_uniform_and_belongs(self):
         point = self.space.random_uniform()
         result = self.space.belongs(point)
-        expected = gs.array([[True]])
+        expected = True
 
         self.assertAllClose(result, expected)
 
     def test_random_uniform(self):
         result = self.space.random_uniform()
 
-        self.assertAllClose(gs.shape(result), (1, self.dimension + 1))
+        self.assertAllClose(gs.shape(result), (self.dimension + 1,))
+
+    def test_projection_to_tangent_space(self):
+        base_point = gs.array([1., 0., 0., 0.])
+        self.assertTrue(self.space.belongs(base_point))
+
+        tangent_vec = self.space.projection_to_tangent_space(
+            vector=gs.array([1., 2., 1., 3.]),
+            base_point=base_point)
+
+        result = self.metric.inner_product(tangent_vec, base_point)
+        expected = 0.
+
+        self.assertAllClose(result, expected)
+
+        result = self.space.projection_to_tangent_space(
+            vector=gs.array([1., 2., 1., 3.]),
+            base_point=base_point)
+        expected = tangent_vec
+
+        self.assertAllClose(result, expected)
 
     def test_intrinsic_and_extrinsic_coords(self):
         """
@@ -43,15 +66,15 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         gives the identity.
         """
         point_int = gs.ones(self.dimension)
-        point_ext = self.space.intrinsic_to_extrinsic_coords(point_int)
-        result = self.space.extrinsic_to_intrinsic_coords(point_ext)
+        point_ext = self.space.from_coordinates(point_int, 'intrinsic')
+        result = self.space.to_coordinates(point_ext, 'intrinsic')
         expected = point_int
         expected = helper.to_vector(expected)
         self.assertAllClose(result, expected)
 
         point_ext = gs.array([2.0, 1.0, 1.0, 1.0])
-        point_int = self.space.extrinsic_to_intrinsic_coords(point_ext)
-        result = self.space.intrinsic_to_extrinsic_coords(point_int)
+        point_int = self.space.to_coordinates(point_ext, 'intrinsic')
+        result = self.space.from_coordinates(point_int, 'intrinsic')
         expected = point_ext
         expected = helper.to_vector(expected)
 
@@ -70,8 +93,8 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
                               [-0.1, .1, -.4, .1, -.01, 0.],
                               [0., 0., .1, .1, -0.08, -0.1],
                               [.1, .1, .1, .1, 0., -0.5]])
-        point_ext = self.space.intrinsic_to_extrinsic_coords(point_int)
-        result = self.space.extrinsic_to_intrinsic_coords(point_ext)
+        point_ext = self.space.from_coordinates(point_int, 'intrinsic')
+        result = self.space.to_coordinates(point_ext, 'intrinsic')
         expected = point_int
         expected = helper.to_vector(expected)
 
@@ -80,8 +103,8 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         point_ext = gs.array([[2., 1., 1., 1.],
                               [4., 1., 3., math.sqrt(5.)],
                               [3., 2., 0., 2.]])
-        point_int = self.space.extrinsic_to_intrinsic_coords(point_ext)
-        result = self.space.intrinsic_to_extrinsic_coords(point_int)
+        point_int = self.space.to_coordinates(point_ext, 'intrinsic')
+        result = self.space.from_coordinates(point_int, 'intrinsic')
         expected = point_ext
         expected = helper.to_vector(expected)
 
@@ -89,8 +112,8 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
 
     def test_log_and_exp_general_case(self):
         """
-        Test that the riemannian exponential
-        and the riemannian logarithm are inverse.
+        Test that the Riemannian exponential
+        and the Riemannian logarithm are inverse.
 
         Expect their composition to give the identity function.
         """
@@ -102,24 +125,71 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         log = self.metric.log(point=point, base_point=base_point)
 
         result = self.metric.exp(tangent_vec=log, base_point=base_point)
-        expected = helper.to_vector(point)
+        expected = point
+        self.assertAllClose(result, expected)
+
+    def test_log_and_exp_general_case_general_dim(self):
+        """
+        Test that the Riemannian exponential
+        and the Riemannian logarithm are inverse.
+
+        Expect their composition to give the identity function.
+        """
+        # Riemannian Log then Riemannian Exp
+        dim = 5
+        n_samples = self.n_samples
+
+        h5 = Hyperboloid(dim=dim)
+        h5_metric = h5.metric
+
+        base_point = h5.random_uniform()
+        point = h5.random_uniform()
+
+        one_log = h5_metric.log(point=point, base_point=base_point)
+
+        result = h5_metric.exp(tangent_vec=one_log, base_point=base_point)
+        expected = point
+        self.assertAllClose(result, expected)
+
+        # Test vectorization of log
+        base_point = gs.stack([base_point] * n_samples, axis=0)
+        point = gs.stack([point] * n_samples, axis=0)
+        expected = gs.stack([one_log] * n_samples, axis=0)
+
+        log = h5_metric.log(point=point, base_point=base_point)
+        result = log
+
+        self.assertAllClose(gs.shape(result), (n_samples, dim + 1))
+        self.assertAllClose(result, expected)
+
+        result = h5_metric.exp(tangent_vec=log, base_point=base_point)
+        expected = point
+
+        self.assertAllClose(gs.shape(result), (n_samples, dim + 1))
+        self.assertAllClose(result, expected)
+
+        # Test vectorization of exp
+        tangent_vec = gs.stack([one_log] * n_samples, axis=0)
+        exp = h5_metric.exp(tangent_vec=tangent_vec, base_point=base_point)
+        result = exp
+
+        expected = point
+        self.assertAllClose(gs.shape(result), (n_samples, dim + 1))
         self.assertAllClose(result, expected)
 
     def test_exp_and_belongs(self):
-        H2 = Hyperbolic(dimension=2)
+        H2 = Hyperboloid(dim=2)
         METRIC = H2.metric
 
         base_point = gs.array([1., 0., 0.])
-        with self.session():
-            self.assertTrue(gs.eval(H2.belongs(base_point)))
+        self.assertTrue(H2.belongs(base_point))
 
         tangent_vec = H2.projection_to_tangent_space(
             vector=gs.array([1., 2., 1.]),
             base_point=base_point)
         exp = METRIC.exp(tangent_vec=tangent_vec,
                          base_point=base_point)
-        with self.session():
-            self.assertTrue(gs.eval(H2.belongs(exp)))
+        self.assertTrue(H2.belongs(exp))
 
     @geomstats.tests.np_and_pytorch_only
     def test_exp_vectorization(self):
@@ -139,7 +209,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         one_tangent_vec = self.space.projection_to_tangent_space(
             one_vec, base_point=one_base_point)
         result = self.metric.exp(one_tangent_vec, one_base_point)
-        self.assertAllClose(gs.shape(result), (1, dim))
+        self.assertAllClose(gs.shape(result), (dim,))
 
         n_tangent_vecs = self.space.projection_to_tangent_space(
             n_vecs, base_point=one_base_point)
@@ -148,12 +218,10 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
 
         expected = gs.zeros((n_samples, dim))
 
-        with self.session():
-            for i in range(n_samples):
-                expected[i] = gs.eval(
-                    self.metric.exp(n_tangent_vecs[i], one_base_point))
-            expected = helper.to_vector(gs.array(expected))
-            self.assertAllClose(result, expected)
+        for i in range(n_samples):
+            expected[i] = self.metric.exp(n_tangent_vecs[i], one_base_point)
+        expected = helper.to_vector(gs.array(expected))
+        self.assertAllClose(result, expected)
 
         one_tangent_vec = self.space.projection_to_tangent_space(
             one_vec, base_point=n_base_points)
@@ -161,12 +229,10 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
         expected = gs.zeros((n_samples, dim))
-        with self.session():
-            for i in range(n_samples):
-                expected[i] = gs.eval(self.metric.exp(one_tangent_vec[i],
-                                      n_base_points[i]))
-            expected = helper.to_vector(gs.array(expected))
-            self.assertAllClose(result, expected)
+        for i in range(n_samples):
+            expected[i] = self.metric.exp(one_tangent_vec[i], n_base_points[i])
+        expected = helper.to_vector(gs.array(expected))
+        self.assertAllClose(result, expected)
 
         n_tangent_vecs = self.space.projection_to_tangent_space(
             n_vecs, base_point=n_base_points)
@@ -174,12 +240,10 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         self.assertAllClose(gs.shape(result), (n_samples, dim))
 
         expected = gs.zeros((n_samples, dim))
-        with self.session():
-            for i in range(n_samples):
-                expected[i] = gs.eval(self.metric.exp(n_tangent_vecs[i],
-                                      n_base_points[i]))
-            expected = helper.to_vector(gs.array(expected))
-            self.assertAllClose(result, expected)
+        for i in range(n_samples):
+            expected[i] = self.metric.exp(n_tangent_vecs[i], n_base_points[i])
+        expected = helper.to_vector(gs.array(expected))
+        self.assertAllClose(result, expected)
 
     def test_log_vectorization(self):
         n_samples = 3
@@ -196,7 +260,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
             [1.0, 0.0, 0.0, 0.0]])
 
         result = self.metric.log(one_point, one_base_point)
-        self.assertAllClose(gs.shape(result), (1, dim))
+        self.assertAllClose(gs.shape(result), (dim,))
 
         result = self.metric.log(n_points, one_base_point)
         self.assertAllClose(gs.shape(result), (n_samples, dim))
@@ -230,8 +294,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         expected = minkowski_space.metric.inner_product(
             tangent_vec_a, tangent_vec_b, base_point)
 
-        with self.session():
-            self.assertAllClose(result, expected)
+        self.assertAllClose(result, expected)
 
     def test_squared_norm_and_squared_dist(self):
         """
@@ -244,8 +307,7 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         result = self.metric.squared_norm(vector=log)
         expected = self.metric.squared_dist(point_a, point_b)
 
-        with self.session():
-            self.assertAllClose(result, expected)
+        self.assertAllClose(result, expected)
 
     def test_norm_and_dist(self):
         """
@@ -258,13 +320,12 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         result = self.metric.norm(vector=log)
         expected = self.metric.dist(point_a, point_b)
 
-        with self.session():
-            self.assertAllClose(result, expected)
+        self.assertAllClose(result, expected)
 
     def test_log_and_exp_edge_case(self):
         """
-        Test that the riemannian exponential
-        and the riemannian logarithm are inverse.
+        Test that the Riemannian exponential
+        and the Riemannian logarithm are inverse.
 
         Expect their composition to give the identity function.
         """
@@ -272,25 +333,24 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         # Edge case: two very close points, base_point_2 and point_2,
         # form an angle < epsilon
         base_point_intrinsic = gs.array([1., 2., 3.])
-        base_point = self.space.intrinsic_to_extrinsic_coords(
-            base_point_intrinsic)
+        base_point =\
+            self.space.from_coordinates(base_point_intrinsic, 'intrinsic')
         point_intrinsic = (base_point_intrinsic +
                            1e-12 * gs.array([-1., -2., 1.]))
-        point = self.space.intrinsic_to_extrinsic_coords(
-            point_intrinsic)
+        point =\
+            self.space.from_coordinates(point_intrinsic, 'intrinsic')
 
         log = self.metric.log(point=point, base_point=base_point)
         result = self.metric.exp(tangent_vec=log, base_point=base_point)
         expected = point
 
-        with self.session():
-            self.assertAllClose(result, expected)
+        self.assertAllClose(result, expected)
 
     @geomstats.tests.np_and_tf_only
     def test_exp_and_log_and_projection_to_tangent_space_general_case(self):
         """
-        Test that the riemannian exponential
-        and the riemannian logarithm are inverse.
+        Test that the Riemannian exponential
+        and the Riemannian logarithm are inverse.
 
         Expect their composition to give the identity function.
         """
@@ -305,58 +365,15 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         result = self.metric.log(point=exp, base_point=base_point)
 
         expected = vector
-        with self.session():
-            self.assertAllClose(result, expected)
+        self.assertAllClose(result, expected)
 
     def test_dist(self):
         # Distance between a point and itself is 0.
         point_a = gs.array([4.0, 1., 3.0, math.sqrt(5)])
         point_b = point_a
         result = self.metric.dist(point_a, point_b)
-        expected = gs.array([[0]])
-
-        with self.session():
-            self.assertAllClose(result, expected)
-
-    @geomstats.tests.np_and_pytorch_only
-    def test_dist_poincare(self):
-
-        point_a = gs.array([0.5, 0.5])
-        point_b = gs.array([0.5, -0.5])
-
-        self.space.metric.point_type = 'ball'
-
-        dist_a_b = self.metric.dist(point_a, point_b)
-        self.space.metric.point_type = 'extrinsic'
-
-        result = dist_a_b
-        expected = gs.array([[2.887270927429199]])
-
-        with self.session():
-            self.assertAllClose(result, expected)
-
-    def test_exp_poincare(self):
-
-        self.space.metric.point_type = 'ball'
-        result = 0
         expected = 0
-        self.space.metric.point_type = 'extrinsic'
-        with self.session():
-            self.assertAllClose(result, expected)
-
-    @geomstats.tests.np_only
-    def test_log_poincare(self):
-
-        point = gs.array([0.3, 0.5])
-        base_point = gs.array([0.3, 0.3])
-
-        self.space.metric.point_type = 'ball'
-        result = self.space.metric.log(point, base_point)
-        expected = gs.array([-0.01733576, 0.21958634])
-
-        self.space.metric.point_type = 'extrinsic'
-        with self.session():
-            self.assertAllClose(result, expected)
+        self.assertAllClose(result, expected)
 
     def test_exp_and_dist_and_projection_to_tangent_space(self):
         base_point = gs.array([4.0, 1., 3.0, math.sqrt(5)])
@@ -364,15 +381,15 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
         tangent_vec = self.space.projection_to_tangent_space(
             vector=vector,
             base_point=base_point)
-        exp = self.metric.exp(tangent_vec=tangent_vec,
-                              base_point=base_point)
+        exp = self.metric.exp(
+            tangent_vec=tangent_vec,
+            base_point=base_point)
 
         result = self.metric.dist(base_point, exp)
         sq_norm = self.metric.embedding_metric.squared_norm(
             tangent_vec)
         expected = sq_norm
-        with self.session():
-            self.assertAllClose(result, expected, atol=1e-2)
+        self.assertAllClose(result, expected, atol=1e-2)
 
     def test_geodesic_and_belongs(self):
         # TODO(nina): Fix this tests, as it fails when geodesic goes "too far"
@@ -389,17 +406,15 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
 
         t = gs.linspace(start=0., stop=1., num=n_geodesic_points)
         points = geodesic(t)
-
         result = self.space.belongs(points)
-        expected = gs.array(n_geodesic_points * [[True]])
+        expected = n_geodesic_points * [True]
 
-        with self.session():
-            self.assertAllClose(expected, result)
+        self.assertAllClose(result, expected)
 
     def test_exp_and_log_and_projection_to_tangent_space_edge_case(self):
         """
-        Test that the riemannian exponential and
-        the riemannian logarithm are inverse.
+        Test that the Riemannian exponential and
+        the Riemannian logarithm are inverse.
 
         Expect their composition to give the identity function.
         """
@@ -419,8 +434,8 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
     @geomstats.tests.np_only
     def test_scaled_inner_product(self):
         base_point_intrinsic = gs.array([1, 1, 1])
-        base_point = self.space.intrinsic_to_extrinsic_coords(
-            base_point_intrinsic)
+        base_point = self.space.from_coordinates(
+            base_point_intrinsic, "intrinsic")
         tangent_vec_a = gs.array([1, 2, 3, 4])
         tangent_vec_b = gs.array([5, 6, 7, 8])
         tangent_vec_a = self.space.projection_to_tangent_space(
@@ -430,8 +445,8 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
             tangent_vec_b,
             base_point)
         scale = 2
-        default_space = Hyperbolic(dimension=self.dimension)
-        scaled_space = Hyperbolic(dimension=self.dimension, scale=2)
+        default_space = Hyperboloid(dim=self.dimension)
+        scaled_space = Hyperboloid(dim=self.dimension, scale=2)
         inner_product_default_metric = \
             default_space.metric.inner_product(
                 tangent_vec_a,
@@ -449,14 +464,14 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
     @geomstats.tests.np_only
     def test_scaled_squared_norm(self):
         base_point_intrinsic = gs.array([1, 1, 1])
-        base_point = self.space.intrinsic_to_extrinsic_coords(
-            base_point_intrinsic)
+        base_point = self.space.from_coordinates(base_point_intrinsic,
+                                                 'intrinsic')
         tangent_vec = gs.array([1, 2, 3, 4])
         tangent_vec = self.space.projection_to_tangent_space(
             tangent_vec, base_point)
         scale = 2
-        default_space = Hyperbolic(dimension=self.dimension)
-        scaled_space = Hyperbolic(dimension=self.dimension, scale=2)
+        default_space = Hyperboloid(dim=self.dimension)
+        scaled_space = Hyperboloid(dim=self.dimension, scale=2)
         squared_norm_default_metric = default_space.metric.squared_norm(
             tangent_vec, base_point)
         squared_norm_scaled_metric = scaled_space.metric.squared_norm(
@@ -469,12 +484,11 @@ class TestHyperbolicMethods(geomstats.tests.TestCase):
     def test_scaled_distance(self):
         point_a_intrinsic = gs.array([1, 2, 3])
         point_b_intrinsic = gs.array([4, 5, 6])
-        point_a = self.space.intrinsic_to_extrinsic_coords(point_a_intrinsic)
-        point_b = self.space.intrinsic_to_extrinsic_coords(point_b_intrinsic)
+        point_a = self.space.from_coordinates(point_a_intrinsic, 'intrinsic')
+        point_b = self.space.from_coordinates(point_b_intrinsic, 'intrinsic')
         scale = 2
-        default_space = Hyperbolic(dimension=self.dimension)
-        scaled_space = Hyperbolic(dimension=self.dimension, scale=2)
-        distance_default_metric = default_space.metric.dist(point_a, point_b)
+        scaled_space = Hyperboloid(dim=self.dimension, scale=2)
+        distance_default_metric = self.space.metric.dist(point_a, point_b)
         distance_scaled_metric = scaled_space.metric.dist(point_a, point_b)
         result = distance_scaled_metric
         expected = scale * distance_default_metric
