@@ -1,6 +1,4 @@
-"""
-Unit tests for Lie groups.
-"""
+"""Unit tests for Lie groups."""
 
 import geomstats.tests
 from geomstats.geometry.lie_group import LieGroup
@@ -8,10 +6,10 @@ from geomstats.geometry.lie_group import LieGroup
 
 class TestLieGroupMethods(geomstats.tests.TestCase):
     dimension = 4
-    group = LieGroup(dimension=dimension)
+    group = LieGroup(dim=dimension)
 
     def test_dimension(self):
-        result = self.group.dimension
+        result = self.group.dim
         expected = self.dimension
 
         self.assertAllClose(result, expected)
