@@ -1,5 +1,6 @@
 """Unit tests for the manifold of symmetric positive definite matrices."""
 
+import math
 import warnings
 
 import geomstats.backend as gs
@@ -14,7 +15,10 @@ from geomstats.geometry.spd_matrices import (
 
 
 class TestSPDMatricesMethods(geomstats.tests.TestCase):
+    """Test of SPDMatrices methods."""
+
     def setUp(self):
+        """Set up the test."""
         warnings.simplefilter('ignore', category=ImportWarning)
 
         gs.random.seed(1234)
@@ -28,22 +32,41 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
         self.n_samples = 4
 
     def test_belongs(self):
+        """Test of belongs method."""
+        mats = gs.array(
+            [[1., 1.], [1., 1.]])
+        result = SPDMatrices.belongs(mats)
+        expected = False
+        self.assertAllClose(result, expected)
+
+    def test_belongs_vectorization(self):
+        """Test of belongs method."""
         mats = gs.array([
-            [[1., 1.], [1., 1.]],
+            [[1., 0], [0, 1.]],
             [[1., 2.], [2., 1.]],
             [[1., 0.], [1., 1.]]])
-        result = SPDMatrices.belongs(mats)
-        expected = gs.array([True, True, False])
+        result = SPDMatrices(2).belongs(mats)
+        expected = gs.array([True, False, False])
         self.assertAllClose(result, expected)
 
     def test_random_uniform_and_belongs(self):
+        """Test of random_uniform and belongs methods."""
+        point = self.space.random_uniform()
+        result = self.space.belongs(point)
+        expected = True
+        self.assertAllClose(result, expected)
+
+    def test_random_uniform_and_belongs_vectorization(self):
+        """Test of random_uniform and belongs methods."""
         points = self.space.random_uniform(4)
         result = self.space.belongs(points)
         expected = gs.array([True] * 4)
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_tf_only
-    def vector_from_symmetric_matrix_and_symmetric_matrix_from_vector(self):
+    @geomstats.tests.np_and_pytorch_only
+    def test_vector_from_symmetric_matrix_and_symmetric_matrix_from_vector(
+            self):
+        """Test for matrix to vector and vector to matrix conversions."""
         sym_mat_1 = gs.array([[1., 0.6, -3.],
                               [0.6, 7., 0.],
                               [-3., 0., 8.]])
@@ -60,8 +83,9 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
 
         self.assertTrue(gs.allclose(result_2, expected_2))
 
-    @geomstats.tests.np_and_tf_only
-    def vector_and_symmetric_matrix_vectorization(self):
+    @geomstats.tests.np_and_pytorch_only
+    def test_vector_and_symmetric_matrix_vectorization(self):
+        """Test of vectorization."""
         n_samples = self.n_samples
         vector = gs.random.rand(n_samples, 6)
         sym_mat = self.space.symmetric_matrix_from_vector(vector)
@@ -77,7 +101,18 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
 
         self.assertTrue(gs.allclose(result, expected))
 
+    def test_logm(self):
+        """Test of logm method."""
+        expected = gs.array([[[0., 1., 0.], [1., 0., 0.], [0., 0., 1.]]])
+        c = math.cosh(1)
+        s = math.sinh(1)
+        e = math.exp(1)
+        v = gs.array([[[c, s, 0.], [s, c, 0.], [0., 0., e]]])
+        result = self.space.logm(v)
+        self.assertAllClose(result, expected)
+
     def test_differential_power(self):
+        """Test of differential_power method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 2.5, 1.5],
                                [0., 1.5, 2.5]])
@@ -89,12 +124,13 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
             power=power,
             tangent_vec=tangent_vec,
             base_point=base_point)
-        expected = gs.array([[[1., 1 / 3, 1 / 3],
-                              [1 / 3, .125, .125],
-                              [1 / 3, .125, .125]]])
+        expected = gs.array([[1., 1 / 3, 1 / 3],
+                             [1 / 3, .125, .125],
+                             [1 / 3, .125, .125]])
         self.assertAllClose(result, expected)
 
     def test_inverse_differential_power(self):
+        """Test of inverse_differential_power method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 2.5, 1.5],
                                [0., 1.5, 2.5]])
@@ -106,13 +142,13 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
             power=power,
             tangent_vec=tangent_vec,
             base_point=base_point)
-        expected = gs.array([[[2., 1., 1.],
+        expected = gs.array([[2., 1., 1.],
                              [1., .5, .5],
-                             [1., .5, .5]]])
+                             [1., .5, .5]])
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_differential_log(self):
+        """Test of differential_log method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 1., 0.],
                                [0., 0., 4.]])
@@ -120,29 +156,30 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
                                 [1., 1., 3.],
                                 [3., 3., 4.]])
         result = self.space.differential_log(tangent_vec, base_point)
-        x = 2 * gs.log(2)
-        expected = gs.array([[[1., 1., x],
-                              [1., 1., x],
-                              [x, x, 1]]])
+        x = 2 * gs.log(2.)
+        expected = gs.array([[1., 1., x],
+                             [1., 1., x],
+                             [x, x, 1]])
+
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_inverse_differential_log(self):
+        """Test of inverse_differential_log method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 1., 0.],
                                [0., 0., 4.]])
-        x = 2 * gs.log(2)
+        x = 2 * gs.log(2.)
         tangent_vec = gs.array([[1., 1., x],
                                 [1., 1., x],
                                 [x, x, 1]])
         result = self.space.inverse_differential_log(tangent_vec, base_point)
-        expected = gs.array([[[1., 1., 3.],
-                              [1., 1., 3.],
-                              [3., 3., 4.]]])
+        expected = gs.array([[1., 1., 3.],
+                             [1., 1., 3.],
+                             [3., 3., 4.]])
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_differential_exp(self):
+        """Test of differential_exp method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 1., 0.],
                                [0., 0., -1.]])
@@ -150,31 +187,32 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
                                 [1., 1., 1.],
                                 [1., 1., 1.]])
         result = self.space.differential_exp(tangent_vec, base_point)
-        x = gs.exp(1)
-        y = gs.sinh(1)
-        expected = gs.array([[[x, x, y],
-                              [x, x, y],
-                              [y, y, 1 / x]]])
+        x = gs.exp(1.)
+        y = gs.sinh(1.)
+        expected = gs.array([[x, x, y],
+                             [x, x, y],
+                             [y, y, 1 / x]])
+
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_inverse_differential_exp(self):
+        """Test of inverse_differential_exp method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 1., 0.],
                                [0., 0., -1.]])
-        x = gs.exp(1)
-        y = gs.sinh(1)
+        x = gs.exp(1.)
+        y = gs.sinh(1.)
         tangent_vec = gs.array([[x, x, y],
                                 [x, x, y],
-                                [y, y, 1 / x]])
+                                [y, y, 1. / x]])
         result = self.space.inverse_differential_exp(tangent_vec, base_point)
-        expected = gs.array([[[1., 1., 1.],
-                              [1., 1., 1.],
-                              [1., 1., 1.]]])
+        expected = gs.array([[1., 1., 1.],
+                             [1., 1., 1.],
+                             [1., 1., 1.]])
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_procrustes_inner_product(self):
+        """Test of SPDMetricProcrustes.inner_product method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 1.5, .5],
                                [0., .5, 1.5]])
@@ -186,12 +224,12 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
                                   [4., 8., 5.]])
         metric = SPDMetricProcrustes(3)
         result = metric.inner_product(tangent_vec_a, tangent_vec_b, base_point)
-        expected = 4
+        expected = gs.array(4.)
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_power_affine_inner_product(self):
+        """Test of SPDMetricAffine.inner_product method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 2.5, 1.5],
                                [0., 1.5, 2.5]])
@@ -200,12 +238,12 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
                                [1., .5, .5]])
         metric = SPDMetricAffine(3, power_affine=.5)
         result = metric.inner_product(tangent_vec, tangent_vec, base_point)
-        expected = [[713 / 144]]
+        expected = 713 / 144
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_power_euclidean_inner_product(self):
+        """Test of SPDMetricEuclidean.inner_product method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 2.5, 1.5],
                                [0., 1.5, 2.5]])
@@ -214,12 +252,13 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
                                 [1., .5, .5]])
         metric = SPDMetricEuclidean(3, power_euclidean=.5)
         result = metric.inner_product(tangent_vec, tangent_vec, base_point)
-        expected = [[3472 / 576]]
+        expected = 3472 / 576
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
+    @geomstats.tests.np_and_tf_only
     def test_euclidean_exp_domain(self):
+        """Test of SPDMetricEuclidean.exp_domain method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 2., 0.],
                                [0., 0., 3.]])
@@ -228,12 +267,12 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
                                 [0., 0., 1.]])
         metric = self.metric_euclidean
         result = metric.exp_domain(tangent_vec, base_point)
-        expected = gs.array([[-3, 1]])
+        expected = gs.array([-3, 1])
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_log_euclidean_inner_product(self):
+        """Test of SPDMetricLogEuclidean.inner_product method."""
         base_point = gs.array([[1., 0., 0.],
                                [0., 1., 0.],
                                [0., 0., 4.]])
@@ -242,13 +281,13 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
                                 [3., 3., 4.]])
         metric = self.metric_logeuclidean
         result = metric.inner_product(tangent_vec, tangent_vec, base_point)
-        x = 2 * gs.log(2)
-        expected = 5. + 4. * x**2
+        x = 2 * gs.log(2.)
+        expected = 5. + 4. * x ** 2
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_tf_only
     def test_log_and_exp_affine_invariant(self):
+        """Test of SPDMetricAffine.log and exp methods with power=1."""
         base_point = gs.array([[5., 0., 0.],
                                [0., 7., 2.],
                                [0., 2., 8.]])
@@ -263,8 +302,8 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_log_and_exp_power_affine(self):
+        """Test of SPDMetricAffine.log and exp methods with power!=1."""
         base_point = gs.array([[5., 0., 0.],
                                [0., 7., 2.],
                                [0., 2., 8.]])
@@ -277,8 +316,8 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
         expected = point
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_log_and_exp_logeuclidean(self):
+        """Test of SPDMetricLogEuclidean.log and exp methods."""
         base_point = gs.array([[5., 0., 0.],
                                [0., 7., 2.],
                                [0., 2., 8.]])
@@ -291,10 +330,10 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
         result = metric.exp(tangent_vec=log, base_point=base_point)
         expected = point
 
-        self.assertAllClose(result, expected)
+        self.assertAllClose(result, expected, atol=1e-5)
 
-    @geomstats.tests.np_and_tf_only
     def test_exp_and_belongs(self):
+        """Test of SPDMetricAffine.exp with power=1 and belongs methods."""
         n_samples = self.n_samples
         base_point = self.space.random_uniform(n_samples=1)
         tangent_vec = self.space.random_tangent_vec_uniform(
@@ -306,8 +345,8 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_and_tf_only
     def test_exp_vectorization(self):
+        """Test of SPDMetricAffine.exp with power=1 and vectorization."""
         n_samples = self.n_samples
         one_base_point = self.space.random_uniform(n_samples=1)
         n_base_point = self.space.random_uniform(n_samples=n_samples)
@@ -330,8 +369,8 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
         self.assertAllClose(
             gs.shape(result), (n_samples, self.space.n, self.space.n))
 
-    @geomstats.tests.np_and_tf_only
     def test_log_vectorization(self):
+        """Test of SPDMetricAffine.log with power 1 and vectorization."""
         n_samples = self.n_samples
         one_base_point = self.space.random_uniform(n_samples=1)
         n_base_point = self.space.random_uniform(n_samples=n_samples)
@@ -358,8 +397,8 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
         self.assertAllClose(
             gs.shape(result), (n_samples, self.space.n, self.space.n))
 
-    @geomstats.tests.np_and_tf_only
     def test_geodesic_and_belongs(self):
+        """Test of SPDMetricAffine.geodesic with power 1 and belongs."""
         initial_point = self.space.random_uniform()
         initial_tangent_vec = self.space.random_tangent_vec_uniform(
             n_samples=1, base_point=initial_point)
@@ -376,45 +415,51 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
 
         self.assertAllClose(result, expected)
 
-    @geomstats.tests.np_only
     def test_squared_dist_is_symmetric(self):
+        """Test of SPDMetricAffine.squared_dist (power=1) and is_symmetric."""
         n_samples = self.n_samples
 
         point_1 = self.space.random_uniform(n_samples=1)
         point_2 = self.space.random_uniform(n_samples=1)
+        point_1 = gs.cast(point_1, gs.float64)
+        point_2 = gs.cast(point_2, gs.float64)
 
         metric = self.metric_affine
+
         sq_dist_1_2 = metric.squared_dist(point_1, point_2)
         sq_dist_2_1 = metric.squared_dist(point_2, point_1)
 
-        self.assertAllClose(sq_dist_1_2, sq_dist_2_1)
+        self.assertAllClose(sq_dist_1_2, sq_dist_2_1, atol=1e-3)
 
-        point_1 = self.space.random_uniform(n_samples=1)
         point_2 = self.space.random_uniform(n_samples=n_samples)
+        point_2 = gs.cast(point_2, gs.float64)
 
         sq_dist_1_2 = metric.squared_dist(point_1, point_2)
         sq_dist_2_1 = metric.squared_dist(point_2, point_1)
-
-        self.assertAllClose(sq_dist_1_2, sq_dist_2_1)
+        self.assertAllClose(sq_dist_1_2, sq_dist_2_1, atol=1e-3)
 
         point_1 = self.space.random_uniform(n_samples=n_samples)
         point_2 = self.space.random_uniform(n_samples=1)
+        point_1 = gs.cast(point_1, gs.float64)
+        point_2 = gs.cast(point_2, gs.float64)
 
         sq_dist_1_2 = metric.squared_dist(point_1, point_2)
         sq_dist_2_1 = metric.squared_dist(point_2, point_1)
 
-        self.assertAllClose(sq_dist_1_2, sq_dist_2_1)
+        self.assertAllClose(sq_dist_1_2, sq_dist_2_1, atol=1e-3)
 
         point_1 = self.space.random_uniform(n_samples=n_samples)
         point_2 = self.space.random_uniform(n_samples=n_samples)
+        point_1 = gs.cast(point_1, gs.float64)
+        point_2 = gs.cast(point_2, gs.float64)
 
         sq_dist_1_2 = metric.squared_dist(point_1, point_2)
         sq_dist_2_1 = metric.squared_dist(point_2, point_1)
 
-        self.assertAllClose(sq_dist_1_2, sq_dist_2_1)
+        self.assertAllClose(sq_dist_1_2, sq_dist_2_1, atol=1e-3)
 
-    @geomstats.tests.np_and_tf_only
     def test_squared_dist_vectorization(self):
+        """Test of SPDMetricAffine.squared_dist (power=1) and vectorization."""
         n_samples = self.n_samples
         point_1 = self.space.random_uniform(n_samples=n_samples)
         point_2 = self.space.random_uniform(n_samples=n_samples)
@@ -422,31 +467,31 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
         metric = self.metric_affine
         result = metric.squared_dist(point_1, point_2)
 
-        self.assertAllClose(gs.shape(result), (n_samples, 1))
+        self.assertAllClose(gs.shape(result), (n_samples,))
 
         point_1 = self.space.random_uniform(n_samples=1)
         point_2 = self.space.random_uniform(n_samples=n_samples)
 
         result = metric.squared_dist(point_1, point_2)
 
-        self.assertAllClose(gs.shape(result), (n_samples, 1))
+        self.assertAllClose(gs.shape(result), (n_samples,))
 
         point_1 = self.space.random_uniform(n_samples=n_samples)
         point_2 = self.space.random_uniform(n_samples=1)
 
         result = metric.squared_dist(point_1, point_2)
 
-        self.assertAllClose(gs.shape(result), (n_samples, 1))
+        self.assertAllClose(gs.shape(result), (n_samples,))
 
         point_1 = self.space.random_uniform(n_samples=1)
         point_2 = self.space.random_uniform(n_samples=1)
 
         result = metric.squared_dist(point_1, point_2)
 
-        self.assertAllClose(gs.shape(result), (1, 1))
+        self.assertAllClose(gs.shape(result), ())
 
-    @geomstats.tests.np_only
     def test_parallel_transport_affine_invariant(self):
+        """Test of SPDMetricAffine.parallel_transport method with power=1."""
         n_samples = self.n_samples
         gs.random.seed(1)
         point = self.space.random_uniform(n_samples)
@@ -460,4 +505,4 @@ class TestSPDMatricesMethods(geomstats.tests.TestCase):
         transported = metric.parallel_transport(tan_a, tan_b, point)
         result = metric.norm(transported, end_point)
 
-        self.assertAllClose(expected, result, atol=1e-5)
+        self.assertAllClose(expected, result, atol=1e-4)
